@@ -120,9 +120,13 @@ class MelodicIntervalTeacher(Teacher):
     When this class was created it was used by melodicinterval
     and singinterval.
     """
+    class ConfigureException(Exception):
+        def __init__(self, msg=_("The exercise has to be better configured.")):
+            Exception.__init__(self, msg)
     OK = 0
     ERR_PICKY = 1
     ERR_CONFIGURE = 2
+    no_intervals_str = _("No intervals selected for interval number %i.") 
     def __init__(self, exname):
         Teacher.__init__(self, exname)
         self.m_tonika = None
@@ -130,10 +134,6 @@ class MelodicIntervalTeacher(Teacher):
     def new_question(self, L, H):
         assert isinstance(L, basestring)
         assert isinstance(H, basestring)
-        if self.get_list('ask_for_intervals_0') == []:
-            return self.ERR_CONFIGURE
-        L, H = utils.adjust_low_high_to_irange(L, H,
-                     self.get_list('ask_for_intervals_0'))
 
         if self.m_timeout_handle:
             gobject.source_remove(self.m_timeout_handle)
@@ -145,7 +145,6 @@ class MelodicIntervalTeacher(Teacher):
                 old_toptone = old_tonika + self.m_question[0]
             self.m_P.next_test_question()
             self.m_question = [self.m_P.m_test_questions[self.m_P.m_test_idx]]
-            #FIXME use tone pitch range from preferences window.
             self.m_tonika = mpd.MusicalPitch()
             # Do this loop to make sure two questions in a row does not have
             # the same top or bottom tone.
@@ -167,64 +166,61 @@ class MelodicIntervalTeacher(Teacher):
         last_tonika = self.m_tonika
         last_question = self.m_question
         if self.get_bool("lock-to-key"):
+            # We don't have to check the validity of these two variables
+            # because get_int will check that a int value is stored in the
+            # database (and make it an int if it is not), and nComboBox
+            # will make sure the int value is within the correct limits.
             lock_tonic = mpd.MusicalPitch.new_from_int(self.get_int("lock-to-key-note"))
-            lock_scaletype = solfege.utils.random_tonic_and_interval_in_key.key_data.keys()[:][self.get_int("lock-to-key-scaletype")]
-
-        for x in range(10):# we try max 10 times to get a question that
-                           # is different from the last one.
-            if self.get_bool("lock-to-key"):
-                self.m_tonika, i = \
-                    utils.random_tonic_and_interval_in_key(L, H,
-                        self.get_list('ask_for_intervals_0'),
-                        lock_tonic, lock_scaletype)
-            else:
-                self.m_tonika, i = utils.random_tonika_and_interval(L, H,
+            lock_scaletype = utils.key_data.keys()[:][self.get_int("lock-to-key-scaletype")]
+        
+        def try_make_question():
+            try:
+                if self.get_bool("lock-to-key"):
+                    self.m_tonika, i = \
+                        utils.random_tonic_and_interval_in_key(L, H,
+                            self.get_list('ask_for_intervals_0'),
+                            lock_tonic, lock_scaletype)
+                else:
+                    self.m_tonika, i = utils.random_tonika_and_interval(L, H,
                             self.get_list('ask_for_intervals_0'))
+            except utils.NoPossibleIntervals, e:
+                raise self.ConfigureException(self.no_intervals_str % 1)
+
             self.m_question = [i]
             t = self.m_tonika + i
             for x in range(1, self.get_int('number_of_intervals=1')):
-                if not self.get_list('ask_for_intervals_%i' % x):
-                    return self.ERR_CONFIGURE
+                interval_list = self.get_list('ask_for_intervals_%i' % x)
+                if not interval_list:
+                    raise self.ConfigureException(self.no_intervals_str % (x + 1))
                 if self.get_bool('lock-to-key'):
                     i = utils.random_interval_in_key(t, L, H,
-                            self.get_list('ask_for_intervals_%i' % x),
+                            interval_list,
                             lock_tonic, lock_scaletype)
                     if not i:
-                        return self.ERR_CONFIGURE
+                        raise self.ConfigureException(_("Failed to select random interval number %i because of the configuration of the exercise. Either you have enabled intervals only if one direction, or none of the intervals belong to the key selected when you enabled «Lock to key».") % x)
                 else:
                     i = utils.random_interval(t, L, H,
                             self.get_list('ask_for_intervals_%i' % x))
-                if not i:
-                    # if we can't find an interval that is with the range
-                    # we, find the interval that is closest to the range
-                    # of notes the user want. This mean that the questions
-                    # are not necessarily that random.
-                    low = mpd.MusicalPitch.new_from_int(L)
-                    high = mpd.MusicalPitch.new_from_int(H)
-                    off = 1000
-                    best = None
-                    for interval in self.get_list('ask_for_intervals_%i'%x):
-                        try:
-                            if t + interval > high:
-                                if t + interval - high < off:
-                                    off = t + interval - high
-                                    best = interval
-                            if t + interval < low:
-                                if low - (t + interval) < off:
-                                    off = low - (t + interval)
-                                    best = interval
-                        except ValueError:
-                            return self.ERR_CONFIGURE
-                    i = best
+                    if not i:
+                        raise self.ConfigureException(_("Failed to select random interval number %i because of the configuration of the exercise. You should select some intervals going in both directions.") % x)
                 self.m_question.append(i)
                 t = t + i
+        max_try = 1000
+        for counter in range(max_try):
+            try:
+                try_make_question()
+            except self.ConfigureException, e:
+                if counter == max_try -1:
+                    raise
+                else:
+                    continue
             if last_tonika is not None \
                     and last_tonika == self.m_tonika \
                     and last_question == self.m_question:
                 continue
-            break
-        self.q_status = self.QSTATUS_NEW
-        return self.OK
+            self.q_status = self.QSTATUS_NEW
+            return self.OK
+
     def play_question(self):
         if self.q_status == self.QSTATUS_NO:
             return
@@ -564,6 +560,8 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         """
         self.handle_config_box_visibility()
         self.handle_statistics_page_sensibility()
+        if self.get_bool("lock-to-key"):
+            self.set_bool("lock-to-key", False)
         if getattr(self, 'g_flashbar', None):
             self.g_flashbar.require_size(["|Xy"])
     def on_end_practise(self):
@@ -839,11 +837,9 @@ class IntervalGui(Gui):
             [mpd.MusicalPitch.new_from_int(60 + x).get_user_notename() for x in range(12)])
         self.g_notename.show()
         self.g_lock_to_key_hbox.pack_start(self.g_notename, False)
-        self.g_scaletype = gu.nComboBox(self.m_exname, 'lock-to-key-scaletype', _("Major"),
-            [_("Major"), _("Natural Minor"), _("Harmonic Minor")])
+        self.g_scaletype = gu.nComboBox(self.m_exname, 'lock-to-key-scaletype', _("Major"), [n['name'] for n in utils.key_data.values()])
         self.g_scaletype.show()
         self.g_lock_to_key_hbox.pack_start(self.g_scaletype, False)
-        self.g_lock_to_key_hbox.set_no_show_all(True)
         toggle_lock_to_key_sensitivity(check)
     def select_inputwidget(self):
         """
