@@ -141,6 +141,8 @@ def iterate_sections():
 def set_string(key, val):
     _maybe_create_key(key)
     section, k = split(key)
+    if not isinstance(val, unicode):
+        val = val.decode("utf-8")
     oldval = get_string(key)
     data[section][k] = val
     if key in _watches and (oldval != get_string(key)):
